@@ -13,6 +13,8 @@ from urllib.request import Request, urlopen
 
 
 if TYPE_CHECKING:
+    from typing import Any
+
     from mkdocs_include_markdown_plugin.cache import Cache
 
 
@@ -112,7 +114,7 @@ MARKDOWN_LINK_DEFINITION_REGEX = re.compile(
 )
 
 
-def transform_p_by_p_skipping_codeblocks(
+def transform_p_by_p_skipping_codeblocks(  # noqa: PLR0912, PLR0915
         markdown: str,
         func: Callable[[str], str],
 ) -> str:
@@ -126,45 +128,80 @@ def transform_p_by_p_skipping_codeblocks(
     _current_fcodeblock_delimiter = ''
 
     # inside indented codeblock
-    _inside_icodeblock = False
+    _maybe_icodeblock_lines: list[str] = []
+    _previous_line_was_empty = False
 
     lines, current_paragraph = ([], '')
 
     def process_current_paragraph() -> None:
         lines.extend(func(current_paragraph).splitlines(keepends=True))
 
+    # The next implementation takes into account that indented code
+    # blocks must be surrounded by newlines as per the CommonMark
+    # specification. See https://spec.commonmark.org/0.28/#indented-code-blocks
+    #
+    # However, note that ambiguities with list items are not handled.
+
     for line in io.StringIO(markdown):
-        if not _current_fcodeblock_delimiter and not _inside_icodeblock:
+        if not _current_fcodeblock_delimiter:
             lstripped_line = line.lstrip()
             if (
                 lstripped_line.startswith('```')
                 or lstripped_line.startswith('~~~')
             ):
                 _current_fcodeblock_delimiter = lstripped_line[:3]
-                if current_paragraph:
+                process_current_paragraph()
+                current_paragraph = ''
+                lines.append(line)
+            elif line.startswith('    '):
+                if not lstripped_line or _maybe_icodeblock_lines:
+                    # maybe enter indented codeblock
+                    _maybe_icodeblock_lines.append(line)
+                else:
+                    current_paragraph += line
+            elif _maybe_icodeblock_lines:
+                process_current_paragraph()
+                current_paragraph = ''
+                if not _previous_line_was_empty:
+                    # wasn't an indented code block
+                    for line_ in _maybe_icodeblock_lines:
+                        current_paragraph += line_
+                    _maybe_icodeblock_lines = []
+                    current_paragraph += line
                     process_current_paragraph()
                     current_paragraph = ''
-                lines.append(line)
-            elif (
-                line.replace('\t', '    ').replace('\r\n', '\n')
-                == '    \n'
-            ):
-                _inside_icodeblock = True
-                if current_paragraph:
-                    process_current_paragraph()
-                    current_paragraph = ''
-                lines.append(line)
+                else:
+                    # exit indented codeblock
+                    for line_ in _maybe_icodeblock_lines:
+                        lines.append(line_)
+                    _maybe_icodeblock_lines = []
+                    lines.append(line)
             else:
                 current_paragraph += line
+            _previous_line_was_empty = not lstripped_line
         else:
             lines.append(line)
-            if _current_fcodeblock_delimiter:
-                if line.lstrip().startswith(_current_fcodeblock_delimiter):
-                    _current_fcodeblock_delimiter = ''
-            elif not line.startswith('    ') and not line.startswith('\t'):
-                _inside_icodeblock = False
+            lstripped_line = line.lstrip()
+            if lstripped_line.startswith(_current_fcodeblock_delimiter):
+                _current_fcodeblock_delimiter = ''
+            _previous_line_was_empty = not lstripped_line
 
-    process_current_paragraph()
+    if _maybe_icodeblock_lines:
+        if not _previous_line_was_empty:
+            # at EOF
+            process_current_paragraph()
+            current_paragraph = ''
+            for line_ in _maybe_icodeblock_lines:
+                current_paragraph += line_
+            process_current_paragraph()
+            current_paragraph = ''
+        else:
+            process_current_paragraph()
+            current_paragraph = ''
+            for line_ in _maybe_icodeblock_lines:
+                lines.append(line_)
+    else:
+        process_current_paragraph()
 
     return ''.join(lines)
 
@@ -178,7 +215,7 @@ def transform_line_by_line_skipping_codeblocks(
     Skip fenced codeblock lines, where the transformation never is applied.
 
     Indented codeblocks are not taken into account because in the practice
-    this function is never used for transformations on indented lines. See
+    this function is only used for transformations of heading prefixes. See
     the PR https://github.com/mondeja/mkdocs-include-markdown-plugin/pull/95
     to recover the implementation handling indented codeblocks.
     """
@@ -240,7 +277,7 @@ def rewrite_relative_urls(
 
     def found_href(m: re.Match[str], url_group_index: int = -1) -> str:
         match_start, match_end = m.span(0)
-        href = m.group(url_group_index)
+        href = m[url_group_index]
         href_start, href_end = m.span(url_group_index)
         rewritten_url = rewrite_url(href)
         return (
@@ -267,6 +304,7 @@ def rewrite_relative_urls(
             functools.partial(found_href, url_group_index=2),
             paragraph,
         )
+
     return transform_p_by_p_skipping_codeblocks(
         markdown,
         transform,
@@ -441,14 +479,53 @@ def read_file(file_path: str, encoding: str) -> str:
         return f.read()
 
 
-def read_url(url: str, http_cache: Cache | None) -> Any:
+def read_url(
+        url: str,
+        http_cache: Cache | None,
+        encoding: str = 'utf-8',
+) -> Any:
     """Read an HTTP location and return its content."""
     if http_cache is not None:
-        cached_content = http_cache.get_(url)
+        cached_content = http_cache.get_(url, encoding)
         if cached_content is not None:
             return cached_content
     with urlopen(Request(url)) as response:
-        content = response.read().decode('UTF-8')
+        content = response.read().decode(encoding)
     if http_cache is not None:
-        http_cache.set_(url, content)
+        http_cache.set_(url, content, encoding)
     return content
+
+
+def safe_os_path_relpath(path: str, start: str) -> str:
+    """Return the relative path of a file from a start directory.
+
+    Safe version of `os.path.relpath` that catches `ValueError` exceptions
+    on Windows and returns the original path in case of error.
+    On Windows, `ValueError` is raised when `path` and `start` are on
+    different drives.
+    """
+    if os.name != 'nt':  # pragma: nt no cover
+        return os.path.relpath(path, start)
+    try:  # pragma: nt cover
+        return os.path.relpath(path, start)
+    except ValueError:  # pragma: no cover
+        return path
+
+
+def file_lineno_message(
+        page_src_path: str | None,
+        docs_dir: str,
+        lineno: int,
+) -> str:
+    """Return a message with the file path and line number."""
+    if page_src_path is None:  # pragma: no cover
+        return f'generated page content (line {lineno})'
+    return (
+        f'{safe_os_path_relpath(page_src_path, docs_dir)}'
+        f':{lineno}'
+    )
+
+
+def lineno_from_content_start(content: str, start: int) -> int:
+    """Return the line number of the first line of ``start`` in ``content``."""
+    return content[:start].count('\n') + 1
